#!/usr/bin/perl -w

# Copyright (C) 1999 David Jao <djao@dominia.mit.edu>.
#
# This program is free software; you may redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
# for more details.

package Apache::LimitByIP;

use strict;
use Apache::Constants qw(:common);
use Apache::Log ();
use IPC::Shareable ();

use vars qw(%IPUSAGE);

sub handler {
    my $r = shift;
    return DECLINED unless $r->is_initial_req;
    my $limit = $r->dir_config('LimitPerIP') || 0;
		# number of concurrent downloads allowed
    return OK if $limit <= 0;            # 0 means unlimited access

# Here's an example of how to limit access to only certain file types.
# This example happens to be particularly useful for mp3 servers ^_^
#    my $content_type = $r->lookup_uri($r->uri)->content_type;
#    return OK unless $content_type =~ m:mpeg:i;

    tie %IPUSAGE, 'IPC::Shareable', 'IPLIMIT', {create => 1, mode => 0644}
    unless defined %IPUSAGE;
    
    my($ip) = ($r->connection->remote_ip);
    
    tied(%IPUSAGE)->shlock;
    
    my($hits) = $IPUSAGE{$ip};
    my $return_value = OK;
    $hits++;
    if ($hits > $limit) { # if current download count exceeds the limit
	$r->server->log->debug("Current download count is ", $hits , ". Denying.");
	$return_value = FORBIDDEN;
    }
    $r->log_reason("Client exceeded download limit.", $r->filename) 
	if $return_value == FORBIDDEN;
    $IPUSAGE{$ip} = $hits;
    tied(%IPUSAGE)->shunlock;
    
    $r->push_handlers(PerlCleanupHandler => \&cleanup);

    return $return_value;
}
sub cleanup {
    my $r = shift;
    return DECLINED unless $r->is_initial_req;  # don't handle sub-requests

# If you pay attention to content type up there, you had better pay
# the same attention down here!
#    my $content_type = $r->lookup_uri($r->uri)->content_type;
#    return OK unless $content_type =~ m:mpeg:i;

  # Not strictly necessary, since the hash should have been created up above
    tie %IPUSAGE, 'IPC::Shareable', 'IPLIMIT', {create => 1, mode => 0644}
    unless defined %IPUSAGE;

    my($ip) = ($r->connection->remote_ip);
    tied(%IPUSAGE)->shlock;
    my($hits) = $IPUSAGE{$ip};
    $hits--;
    $r->server->log->debug("Download count decremented to ", $hits , ".");
    $IPUSAGE{$ip} = $hits;
    delete $IPUSAGE{$ip} if $hits = 0;
      # cleaning up keys does wonders for your memory usage
    tied(%IPUSAGE)->shunlock;
}

1;
__END__

=head1 NAME

Apache::LimitByIP - Limit simultaneous connections by IP address

=head1 SYNOPSIS

 <Location /somewhere> 
 PerlAccessHandler Apache::LimitByIP
 PerlSetVar        LimitPerIP 3
 </Location>

=head1 DESCRIPTION

The Apache::LimitByIP module lets you enforce limits on the number of
simultaneous downloads allowed from a single IP address. The LimitPerIP
variable dictates the maximum number of simultaneous connections that
each IP address is allowed. Connections in excess of this limit will
result in a 403 Forbidden response.

The default value of 0 for LimitPerIP allows unlimited simultaneous
connections.

Note that the value of LimitPerIP is global throughout the entire
server, so you aren't allowed to set different values of LimitPerIP
for different Locations. Also, the limit defined in LimitPerIP applies
to all IP addresses connecting to your Apache server. Currently there
is no way to set different limits for different IP addresses.

=head1 SEE ALSO

Apache(3), IPC::Shareable(3)

=head1 CREDITS

This module borrows ideas from Apache::SpeedLimit in the mod_perl
book (http://www.modperl.com/), most notably the technique of
storing persistent state information in shared memory.

=head1 AUTHOR

David Jao <djao@dominia.mit.edu>

=head1 NOTES

This module requires IPC::Shareable and has only been tested on Linux 2.x.
